/************************************************************************/
/* This module handles reading and writing of data to the extended disc */
/* image type.                                                          */
/************************************************************************/


/* Code specific to disk images */


/* Extended disk image code */

#include "extdsk.h"
#include "dskstruc.h"
#include "filexfer.h"

#define min(a,b) ((a)<(b)) ? (a) : (b)

static void     GetTrackHeader(DSK_UNIT_DATA *,int,int);
/*static int      GetSectorSize(int);*/
static int      GetSectorPos(EXTDSKTRACKHEADER *,int);


/*static int      DskSectorSizeTable[7]={128,256,512,1024,2048,4096,8192};
*/

int     ExtOpenDiskImage(DSK_UNIT_DATA *pDskUnit,char *DskFilename)
{
int     i;
int     TrackOffset;
EXTDSKHEADER    *pDskHeader;

        /* attempt to open disk image */

        if ((pDskUnit->DiskImageFileHandle=fopen(DskFilename,"rb+"))==NULL)
                return FALSE;

        /* read header into memory */

        fseek(pDskUnit->DiskImageFileHandle,0,SEEK_SET);
        fread(&pDskUnit->DskHeader[0],sizeof(EXTDSKHEADER),1,pDskUnit->DiskImageFileHandle);

        /* identify if disk image is standard or extended */



        /* Setup data for disk image format found */

        TrackOffset = sizeof(EXTDSKHEADER);

        pDskHeader = (EXTDSKHEADER *)&pDskUnit->DskHeader[0];

        for (i=0; i<255-4-14-33; i++)
        {
                if (pDskHeader->TrackSizeTable[i] != 0)
                {

                        pDskUnit->TrackOffsetTable[i] = TrackOffset;
                        TrackOffset += (int)((pDskHeader->TrackSizeTable[i] & 0x0ff)<<8);
                }
                else
                {
                        pDskUnit->TrackOffsetTable[i] = 0;
                }
        }

        pDskUnit->NumSides = pDskHeader->NumSides;

        return TRUE;
}

static int GetTrackOffset(DSK_UNIT_DATA *pDskUnit,int Track,int Side)
{
	EXTDSKHEADER            *pDskHeader;
	int nIndex = Track;
        pDskHeader = (EXTDSKHEADER *)&pDskUnit->DskHeader[0];
	
	if (pDskHeader->NumSides!=1)
	{
		nIndex = (Track*2)+(Side & 1);
	}
		
	return pDskUnit->TrackOffsetTable[nIndex];
}

static void     GetTrackHeader(DSK_UNIT_DATA *pDskUnit,int Track,int Side)
{
        pDskUnit->TrackOffset=GetTrackOffset(pDskUnit, Track, Side);
	
        fseek(pDskUnit->DiskImageFileHandle,(long)pDskUnit->TrackOffset,SEEK_SET);
        fread(&pDskUnit->DskTrackHeader[0],sizeof(EXTDSKTRACKHEADER),1,pDskUnit->DiskImageFileHandle);
}

/* Return sector size from N value in sector ID */

/*static int      GetSectorSize(int N)
{
        if (N<8)
                return DskSectorSizeTable[N];
        else
                return 0x0200;
}*/

/* Returns position 0->15 or -1 if error */

static int      GetSectorPos(EXTDSKTRACKHEADER *pDskTrackHeader,int Sector)
{
int     i;
int     SectorPos = 0;

        for (i=0; i<pDskTrackHeader->SPT; i++)
        {
                if (pDskTrackHeader->SectorIDs[i].R==Sector)
                        return SectorPos;

                SectorPos++;

        }
        return -1;
}

int     ExtGetSectorDsk(DSK_UNIT_DATA *pDskUnit,SECTOR_POS *pSectorPos, char *pData, int Length)
{
int     SectorPos;
EXTDSKTRACKHEADER *pDskTrackHeader;
int     i;
int     SectorSize;
int CopyLength;

        /* Check sector exists */
        GetTrackHeader(pDskUnit,pSectorPos->Track,pSectorPos->Side);

        pDskTrackHeader = (EXTDSKTRACKHEADER *)&pDskUnit->DskTrackHeader[0];

	SectorPos = GetSectorPos(pDskTrackHeader,pSectorPos->R);
        if (SectorPos==-1)
        {
                #ifdef DEBUG
                printf("Sector not found!\n");
                #endif

                return FALSE;
        }

        SectorSize = (pDskTrackHeader->SectorIDs[SectorPos].SectorSizeHigh<<8)|pDskTrackHeader->SectorIDs[SectorPos].SectorSizeLow;

        pDskUnit->SectorOffset = sizeof(EXTDSKTRACKHEADER);
        for (i=0; i<SectorPos; i++)
        {
                pDskUnit->SectorOffset +=((pDskTrackHeader->SectorIDs[i].SectorSizeHigh & 0x0ff)<<8)|(pDskTrackHeader->SectorIDs[i].SectorSizeLow & 0x0ff);
        }

    if (Length!=-1)
    {
        CopyLength = min(SectorSize, Length);
    }
    else
    {
        CopyLength = SectorSize;
    }
	if (CopyLength!=0)
	{
		pDskUnit->SectorOffset += pDskUnit->TrackOffset;
		fseek(pDskUnit->DiskImageFileHandle,(long)pDskUnit->SectorOffset,SEEK_SET);
		fread(pData,CopyLength,1,pDskUnit->DiskImageFileHandle);
	}
	
        return TRUE;
}

int     ExtPutSectorDsk(DSK_UNIT_DATA *pDskUnit,SECTOR_POS *pSectorPos, char *pData, int Length)
{
int     SectorPos;
EXTDSKTRACKHEADER *pDskTrackHeader;
int     i;
int     SectorSize;
int CopyLength;

        /* Check sector exists */

        GetTrackHeader(pDskUnit,pSectorPos->Track,pSectorPos->Side);

        pDskTrackHeader = (EXTDSKTRACKHEADER *)&pDskUnit->DskTrackHeader[0];

        if ((SectorPos = GetSectorPos(pDskTrackHeader,pSectorPos->R))==-1)
        {
                #ifdef DEBUG
                printf("Sector not found!\n");
                #endif

                return FALSE;
        }

        SectorSize = (pDskTrackHeader->SectorIDs[SectorPos].SectorSizeHigh<<8)|pDskTrackHeader->SectorIDs[SectorPos].SectorSizeLow;

        pDskUnit->SectorOffset = sizeof(EXTDSKTRACKHEADER);

        for (i=0; i<SectorPos; i++)
        {
                pDskUnit->SectorOffset +=((pDskTrackHeader->SectorIDs[i].SectorSizeHigh & 0x0ff)<<8)|(pDskTrackHeader->SectorIDs[i].SectorSizeLow & 0x0ff);
        }
  if (Length!=-1)
    {
        CopyLength = min(SectorSize, Length);
    }
    else
    {
        CopyLength = SectorSize;
    }
        pDskUnit->SectorOffset += pDskUnit->TrackOffset;
        fseek(pDskUnit->DiskImageFileHandle,(long)pDskUnit->SectorOffset,SEEK_SET);
        fwrite(pData,CopyLength,1,pDskUnit->DiskImageFileHandle);

        return TRUE;
}

int     ExtGetFormatDsk(DSK_UNIT_DATA *pDskUnit,FORMAT_DATA *pFormatData)
{
EXTDSKTRACKHEADER       *pDskTrackHeader;
EXTDSKHEADER            *pDskHeader;

        /* Temporary */

        GetTrackHeader(pDskUnit,0,0);

        pDskTrackHeader = (EXTDSKTRACKHEADER *)&pDskUnit->DskTrackHeader[0];
        pDskHeader = (EXTDSKHEADER *)&pDskUnit->DskHeader[0];

        pFormatData->FirstSectorID = pDskTrackHeader->SectorIDs[0].R & 0x0ff;
        pFormatData->SectorNSize = pDskTrackHeader->BPS & 0x0ff;
        pFormatData->NumberOfSides = pDskHeader->NumSides;
        pFormatData->SectorsPerTrack = pDskTrackHeader->SPT;




        return TRUE;
}

