/* WIN32 Console functions */

#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>

#include "winconsole.h"

static HANDLE hOutputConsole = 0;
static CONSOLE_SCREEN_BUFFER_INFO ConsoleScreenBufferInfo;
static int ConsoleWidth;
static int ConsoleHeight;

/* initialise console */
void	WinConsole_Initialise()
{
	/* get output console */
	hOutputConsole = GetStdHandle(STD_OUTPUT_HANDLE);

	/* get console screen buffer info */
	GetConsoleScreenBufferInfo(hOutputConsole, &ConsoleScreenBufferInfo);

	/* calculate console width and height */
	ConsoleWidth = ConsoleScreenBufferInfo.srWindow.Right - ConsoleScreenBufferInfo.srWindow.Left;
	ConsoleHeight = ConsoleScreenBufferInfo.srWindow.Bottom - ConsoleScreenBufferInfo.srWindow.Top;

	SetConsoleTextAttribute(hOutputConsole, FOREGROUND_RED | FOREGROUND_BLUE | FOREGROUND_GREEN);
}

void	WinConsole_Clear()
{
	DWORD NoOfCharsWritten = 0;
	COORD	StartCoord;

	StartCoord.X = 0;
	StartCoord.Y = 0;

	FillConsoleOutputCharacter(hOutputConsole,
		' ',
		ConsoleWidth * ConsoleHeight,
		StartCoord,
		&NoOfCharsWritten);

	FillConsoleOutputAttribute(hOutputConsole,
		0,
		ConsoleWidth * ConsoleHeight,
		StartCoord,
		&NoOfCharsWritten);

	WinConsole_SetCursorPosition(0,0);
}




/* set console cursor position */
void	WinConsole_SetCursorPosition(int Column,int Row)
{
	COORD	CursorCoordinates;

	CursorCoordinates.X = Column;
	CursorCoordinates.Y = Row;

	SetConsoleCursorPosition(hOutputConsole, CursorCoordinates);
}

/* set console title */
void	WinConsole_SetConsoleTitle(char *Title)
{
	SetConsoleTitle(Title);
}

void	WinConsole_WriteChar(char ch)
{
	char	Buffer[1];
	DWORD		NoOfCharsWritten = 0;

	Buffer[0] = ch;

	WriteConsole(hOutputConsole, Buffer, 1, &NoOfCharsWritten, NULL);
}

int	WinConsole_GetConsoleWidth()
{
	return ConsoleWidth;
}

int WinConsole_GetConsoleHeight()
{
	return ConsoleHeight;
}
