#include <stdio.h>
#include <stdlib.h>
#include "filexfer/filexfer.h"
#ifdef UNIX
#include <sys/io.h>
#else
#include <io.h>
#endif
#include "getopt.h"
#include <string.h>

extern char *optarg;



int ReadNumberParameter(char *param)
{
    int Length = strlen(param);
    BOOL bIsHex = FALSE;
    int Offset = 0;
    unsigned long Value = 0;
    char ch;

    if (Length==0)
        return 0;

    /* check for common prefixs for hex numbers */
    if ((Length>1) && ((param[0]=='&') || (param[0]=='$')))
    {
        Offset = 1;
        bIsHex = TRUE;
    }
    else if ((Length>2) && (param[0]=='0') && ((param[1]=='x') || (param[1]=='X')))
    {
        Offset = 2;
        bIsHex = TRUE;
    }

    if (!bIsHex)
    {
        return atoi(param);
    }

    ch = param[Offset];
    while (ch!='\0')
    {
        Value = Value<<4;
        if ((ch>='0') && (ch<='9'))
        {
            Value = Value | (ch-'0');
        }
        else if ((ch>='a') && (ch<='f'))
        {
            Value = Value | ((ch-'a')+10);
        }
        else if ((ch>='A') && (ch<='F'))
        {
            Value = Value | ((ch-'A')+10);
        }
        Offset++;
        ch = param[Offset];
    }

    return Value;
}

enum
{
    OPERATION_READ_SECTORS = 0,
    OPERATION_WRITE_SECTORS
};

int main(int argc, char **argv)
{
    FILE *fh;
    int StartPhysicalTrack = 2;
    int StartPhysicalSide = 0;
    int StartC = 0;
    int StartH = 0;
    int StartR = 0x0c5;
    int StartN = 0x02;
    int nOverrideLength = 0;
    BOOL bOverrideLengthDefined = FALSE;
    int Operation = OPERATION_READ_SECTORS;
    char c;

// t = Physical Track
// h = Physical Side
// o = operation

     do
    {
        c = getopt(argc, argv,"t:s:o:i:l:");

        switch (c)
        {
            case 't':
            {
                StartPhysicalTrack = ReadNumberParameter(optarg) & 0x0ff;
            }
            break;


            case 's':
            {
                StartPhysicalSide = ReadNumberParameter(optarg) & 0x01;
            }
            break;

            case 'i':
            {
                StartR = ReadNumberParameter(optarg) & 0x0ff;
            }
            break;

            case 'l':
            {
                nOverrideLength = ReadNumberParameter(optarg);
                bOverrideLengthDefined = TRUE;
            }
            break;

            case 'o':
            {
                Operation = atoi(optarg);
            }
            break;

            default:
            {
            }
            break;
        }

    }
    while (c!=-1);

    if ((argc-optind)==0)
    {
        printf("No disk image have been specified!\n");
        exit(1);
    }

    if ((argc-optind)==1)
    {
        printf("No file has been specified (for read/write)\n");
        exit(1);
    }

    printf("Opening DSK \"");
    printf("%s",argv[optind]);
    printf("\"\n");

    FileXfer_InitialiseDrive(0, DISK_IMAGE, argv[optind]);

    if (Operation==OPERATION_WRITE_SECTORS)
    {
	    printf("Operation: Write sectors\n");

	    printf("Physical starting track: %d\n", StartPhysicalTrack);
	    printf("Physical starting side: %d\n", StartPhysicalSide);

        fh = fopen(argv[optind+1], "rb");
        if (fh!=NULL)
        {
            int Length;

            printf("Reading file \"");
	        printf("%s",argv[optind+1]);
	        printf("\"\n");

            SECTOR_POS SectorPos;
            char *pData = NULL;

            fseek(fh, 0, SEEK_END);
            Length = ftell(fh);
            fseek(fh, 0, SEEK_SET);

            if (bOverrideLengthDefined)
            {
                if (nOverrideLength<Length)
                {
                    Length = nOverrideLength;
                }
            }
            printf("Length: %d\n", Length);

            if (Length!=0)
            {
                pData = malloc(Length);

                if (pData!=NULL)
                {
                    int LengthRemaining;
                    int Offset = 0;
                    int SectorSize = 512;
                    int LengthCopy;

                    SectorPos.C = StartC;
                    SectorPos.H = StartH;
                    SectorPos.R = StartR;
                    SectorPos.N = StartN;
                    SectorPos.Track = StartPhysicalTrack;
                    SectorPos.Side = StartPhysicalSide;

                    fread(pData, 1, Length, fh);

                    LengthRemaining = Length;

                    while (LengthRemaining!=0)
                    {

                        if (LengthRemaining>SectorSize)
                        {
                            LengthCopy = SectorSize;
                        }
                        else
                        {
                            LengthCopy = LengthRemaining;
                        }
                        printf("Writing sector: Track: %02d Side: %02d (C: &%02x, H: &%02x R: &%02x N: &%02x)\n", SectorPos.Track, SectorPos.Side, SectorPos.C,SectorPos.H,SectorPos.R,SectorPos.N);

                        FileXFer_WriteSector(0, &SectorPos, &pData[Offset], LengthCopy);

                        // get next sector pos
                        SectorPos.R++;
                        if (SectorPos.R>0x0c9)
                        {
                            SectorPos.R=0x0c1;
                            SectorPos.Track++;
                            SectorPos.C++;
                        }
                        Offset+=LengthCopy;

                        LengthRemaining -= LengthCopy;
                    }

                    printf("Writing complete.\n");
                    free(pData);
                }
            }
            fclose(fh);
        }
    }                                                    

    else
    if (Operation==OPERATION_READ_SECTORS)
    {
        printf("Operation: Read sectors\n");

        printf("Physical starting track: %d\n", StartPhysicalTrack);
        printf("Physical starting side: %d\n", StartPhysicalSide);

        fh = fopen(argv[optind+1], "w+b");
        if (fh!=NULL)
        {
            printf("Opened file \"");
	        printf("%s",argv[optind+1]);
	        printf("\" for writing\n");

            SECTOR_POS SectorPos;
            /* allocate space for sector */
            char *pData = malloc(16384);

            if (pData!=NULL)
            {
                int LengthRemaining;
                int Offset = 0;
                int SectorSize = 512;
                int LengthCopy;
                int Length = 0;

                if (bOverrideLengthDefined)
                {
                    Length = nOverrideLength;
                }
                printf("Length: %d\n", Length);

                SectorPos.C = StartC;
                SectorPos.H = StartH;
                SectorPos.R = StartR;
                SectorPos.N = StartN;
                SectorPos.Track = StartPhysicalTrack;
                SectorPos.Side = StartPhysicalSide;

                LengthRemaining = Length;

                while (LengthRemaining!=0)
                {
                    if (LengthRemaining>SectorSize)
                    {
                        LengthCopy = SectorSize;
                    }
                    else
                    {
                        LengthCopy = LengthRemaining;
                    }
                    printf("Reading sector: Track: %02d Side: %02d (C: &%02x, H: &%02x R: &%02x N: &%02x)\n", SectorPos.Track, SectorPos.Side, SectorPos.C,SectorPos.H,SectorPos.R,SectorPos.N);

                    FileXFer_ReadSector(0, &SectorPos, pData, LengthCopy);
                    fwrite(pData, 1, LengthCopy, fh);

                    // get next sector pos
                    SectorPos.R++;
                    if (SectorPos.R>0x0c9)
                    {
                        SectorPos.R=0x0c1;
                        SectorPos.Track++;
                        SectorPos.C++;
                    }
                    Offset+=LengthCopy;

                    LengthRemaining -= LengthCopy;
                }

                printf("Reading complete.\n");
                free(pData);
            }
            fclose(fh);
        }
    }

    FileXfer_CloseFile(0);

    return 0;
}
